
# Partage sous licence Creative Commons :
# https://creativecommons.org/licenses/by-sa/3.0/fr/

autor = 'Grégory COUTABLE'
version = '2.0'
date = '13/01/2023'

import pygame
from pygame.locals import *
#pour le rendre dispo de n'importe où
import os
pathname = os.path.dirname(__file__)

class GUImemory(object):
    """
    Interface graphique du jeu de memory.
    """
    
    def __init__(self, nbx, nby):
       
        #des constantes pour le dessin de la grille
        self.nbx = nbx
        self.nby = nby
        self.w = 600 // self.nby  #largeur d'une case
        if self.w > 200:
            self.w = 200
        self.d = 8 #entre deux cases
        #la liste des coordonnées de chaque case de la grille (coordonnées du coin en haut à gauche)
        self.xy0 = [self.d + (self.d + self.w) * x for x in range(max(self.nbx, self.nby))]

        #initialisation de pygame
        pygame.init()
        #calcul de la taille totale de la fenètre
        self.wFen = self.nbx * self.w + (self.nbx + 1) * self.d
        # hauteur idem + un bandeau pour le texte
        self.hFen = self.nby * self.w + (self.nby + 1) * self.d + 100
        #on définit la fenêtre de base de notre jeu
        self.fond = pygame.display.set_mode((self.wFen, self.hFen))
        #un titre sur cette fenêtre
        pygame.display.set_caption("MEMORY")

        # une liste contenant les différentes cases possibles de vide
        
        self.spriteSheet = pygame.transform.scale(pygame.image.load(os.path.join(pathname, "halloween.png")).convert_alpha(), (20 * self.w, self.w))
        self.cases = [self._creerCase(i) for i in range(21)]
    
    def _creerCase(self, n):
        case = pygame.Surface((self.w, self.w))
        
        if n != 0:
            self.image = self.spriteSheet.subsurface(pygame.Rect((n - 1) * self.w, 0, self.w, self.w))
            case.blit(self.image, case.get_rect())
        else:
            case.fill(pygame.Color('#474455'))
        
        return case
    
    def refresh(self, grille, txt = ''):
        """
        Cette méthode rafraichie l'affichage du jeu de memory conformément à la grille passée en argument.

        parameters
        ----------
        g : liste de nby listes.
            Par exemple ;
            g[0][0] est la case en haut a gauche,
            g[3][3] est la case en bas à droite dans une grille de 4*4.
        
            0 -> case vide
            1 à 20 -> une image
            Attention au dela de 20, cela déclenche une erreur.
            
        txt : str
            Un texte a afficher dans le bandeau

        """

        self.fond.fill(pygame.Color("#BCAF9F"))
        for y in range(self.nby):
            for x in range(self.nbx):
                
                self.fond.blit(self.cases[self._getValue(grille, x, y)], (self.xy0[x], self.xy0[y])) 

        #zone de texte
        police = self._adaptPolice(40, txt, self.wFen - 50)
        texte = police.render(txt, True, '#2B2833')
        
        #Une surface pour centrer le texte sur le bandeau du haut
        surfText = pygame.Rect(0, self.hFen - 100, self.wFen, 100)
        #pour centrer le texte
        rectTexte = texte.get_rect()
        rectTexte.center = surfText.center
        self.fond.blit(texte, rectTexte)

        #Rafraîchissement de l'écran
        pygame.display.update()

    def _getValue(self, grille, x, y):
        """
        Une primitive pour accéder a une valeur dans grille.
        
        parameters
        ----------
        grille : list
            Par défaut, elle est adaptée a une structure type liste de liste.
            A redéfinir si grille a un autre format
        x : int
        y : int
            >=0, coordonnée de la case 0, 0 en haut à gauche.
        """
        value = grille[y][x]
        
        assert value <= 20, "Il n'y a que 20 figures disponibles dans cette GUI"
        return value
        

    def _adaptPolice(self, wPol, txt, w, police = None):
        """
        adaptation automatique de la taille de la police dans la zone de saisie.
        
        Parameters
        ----------
        wPol : int
            taille max de la police
        txt : str
            le texte a afficher
        w : int
            la taille a ne pas dépasser
        police : str
            la police a utiliser
        """
        taille = False
        while not taille:
            if police is not None:
                pol = pygame.font.Font(os.path.join(pathname, police), wPol)
            else:
                pol = pygame.font.SysFont("Arial Black", wPol)
            texte = pol.render(txt, True, 'white')
            if texte.get_width() < w:
                taille = True
            else:
                wPol = int(wPol * 0.8)
        return pol

    def gagne(self, txt):
        """
        Cette méthode permet d'afficher GAME OVER plein écran.
        """
        self.fond.fill("green")
        police = pygame.font.Font(os.path.join(pathname, "led.ttf"), 72)
        texteG = police.render('GAGNÉ !!', True, pygame.Color("black"))
        police = self._adaptPolice(54, txt, self.wFen - 50, "led.ttf")
        texte = police.render(txt, True, pygame.Color("black"))
        #pour centrer le texte
        rectTexte = texte.get_rect()
        rectTexte.center = self.fond.get_rect().center
        rectTexte.top = self.fond.get_rect().center[1]
        self.fond.blit(texte,rectTexte)
        rectTexteG = texteG.get_rect()
        rectTexteG.center = self.fond.get_rect().center
        rectTexteG.bottom = self.fond.get_rect().center[1]        
        
        self.fond.blit(texteG,rectTexteG)
        #Rafraîchissement de l'écran
        pygame.display.update()

    def waitClick(self):
        """
        Cette méthode attend l'action d'un joueur. Elle gère trois type d'actions :
            - demande fermeture de la fenètre : fermeture propre de la fenètre pygame et fin du programme python.
            - click sur la fenetre : retourne un tuple contenant les numéros (x, y) de la case choisie.
            - appui sur des 4 flèches du clavier : retourne un caractère '_U', '_D', '_L', ou '_R'. Quelques autres touches sont également gérées et retourne la lettre saisie.
        Une fois exécutée, on ne peut sortir de cette méthode que par l'une de ces trois actions.

        """        
        while True:
            #Limitation de vitesse de la boucle
            pygame.time.Clock().tick(30)
            
            for event in pygame.event.get():    #Attente des événements
                if event.type == QUIT:
                    pygame.quit()
                    exit()
                    
                if event.type == MOUSEBUTTONDOWN:
                    if event.button == 1:   #Si clic gauche
                        if event.pos[1] < self.nby * (self.w + self.d):
                            return (event.pos[0]//(self.w + self.d), event.pos[1]//(self.w + self.d))

                if event.type == KEYDOWN:
                    touches = {K_RIGHT : '_R', K_LEFT : '_L', K_UP : '_U', K_DOWN : '_D', K_RETURN : '_E', K_BACKSPACE : '_B', K_ESCAPE : '_S'}
                    touche = event.key
                    if touche in touches:
                        return touches[touche]
                    return event.unicode

                          

if __name__ == "__main__":
    import time

    g = [[0 for x in range(5)] for y in range(4)]
    print(g)
    
    GUI = GUImemory(5, 4)
    
    GUI.refresh(g, 'Un texte')
    time.sleep(2)
    GUI.gagne("en 18 coups !!")
    time.sleep(2)

    for y in range(4):
        for x in range(5):
            g[y][x] = y * 5 + x + 1
            GUI.refresh(g, F"{y * 5 + x + 1} images différentes")
            time.sleep(0.1)            

    while True:
        xy = GUI.waitClick()
        GUI.refresh(g, str(xy))